#!/bin/bash

declare SCRIPT_DIR=$( dirname $( readlink -f $0 ) )
source $SCRIPT_DIR/include

declare LINTEL_PACK_BINDIR=/opt/mcst/lintel/bin

function print_help()
{
    if declare -p __SHOW_ACTION__ >& /dev/null
    then
        printf "\nErase or restore lintel on device\n\n"
    else
        printf "\n"
    fi
    printf "Usage:\n"
    printf "$( readlink -f $0 ) [options]\n\n"
    printf "Options:\n"
    printf "\t-h, -H, --help                   print this help\n"
    printf "\t-D <lintel storage device>       set device to erase lintel from (by default trying /dev/hda, /dev/hdc)\n"
    printf "\t-R                               restore on device previously erased lintel (restore mode)\n"
    printf "\t-I                               ask approval on erase or restore (interactive mode), in this mode stdout\n"
    printf "\t                                 must refer to a terminal\n"
    printf "\t-E                               examine whether lintel is present on device (examine mode), in this mode\n"
    printf "\t                                 lintel is neither erased nor restored, options -R and -I are ignored\n"

    return 0
}

# Parse options
declare LINTEL_DEVICE=""
declare IS_EXAMINE_MODE=""
declare IS_RESTORE_MODE=""
declare IS_INTERACTIVE_MODE=""
declare PARSE_OPTIONS_SECTION=
while [ $# != 0 ]
do
    case $1 in
        -h | -H | --help )
            __SHOW_ACTION__= print_help
            exit 0
        ;;
        -D )
            shift
            test $1 || error "no argument for -D option"
            test -b $1 || error "invalid argument for -D option"
            LINTEL_DEVICE="$1"
        ;;
        -E )
            IS_EXAMINE_MODE=1
        ;;
        -I )
            IS_INTERACTIVE_MODE=1
            test -t 1 || error "stdout does not refer to a terminal"
        ;;
        -R )
            IS_RESTORE_MODE=1
        ;;
        * )
            error "unsupported option $1"
        ;;
    esac
    shift
done
unset -v PARSE_OPTIONS_SECTION

# If LINTEL_DEVICE is not set try to use default device
if [ -z "${LINTEL_DEVICE:-""}" ]
then
    if test -b /dev/hda
    then
        LINTEL_DEVICE=/dev/hda
    else
        if test -b /dev/hdc
        then
            LINTEL_DEVICE=/dev/hdc
        else
            error "can not find IDE device to erase lintel image from"
        fi
    fi
fi
__INFO__= info "device $LINTEL_DEVICE will be used"

examine_device $LINTEL_DEVICE

if [ -n "${IS_EXAMINE_MODE:-""}" ]
then

    info "examine mode is selected (-E)"

    if does_device_have_b_lintel_signature
    then
        report "lintel signature detected in the zeroth sector of device $LINTEL_DEVICE"
    fi

    if does_device_have_e_lintel_signature
    then
        report "lintel signature detected in the last sector of device $LINTEL_DEVICE (one disk mode placement)"
    fi

    if ! does_device_have_b_lintel_signature && ! does_device_have_e_lintel_signature
    then
        report "no lintel detected on device $LINTEL_DEVICE"
    fi

    .exit 0

fi

if [ -n "${IS_INTERACTIVE_MODE:-""}" ]
then
    info "interactive mode is selected (-I)"
    alias .is_action_confirmed=is_action_confirmed
else
    alias .is_action_confirmed=": "
fi

if [ -n "${IS_RESTORE_MODE:-""}" ]
then

    : restore

    info "restore mode is selected (-R)"

    if does_device_have_b_erased_signature
    then
        if .is_action_confirmed "Do you want to restore lintel signature in the zeroth sector of device $LINTEL_DEVICE?"
        then
            restore_b_lintel_signature $LINTEL_DEVICE
            report "lintel signature has been restored in the zeroth sector of device $LINTEL_DEVICE"
        else
            warning "lintel signature has not been restored in the zeroth sector of device $LINTEL_DEVICE"
        fi
    fi

    if does_device_have_e_erased_signature
    then
        if .is_action_confirmed "Do you want to restore lintel signature in the last sector of device $LINTEL_DEVICE?"
        then
            restore_e_lintel_signature $LINTEL_DEVICE
            report "lintel signature has been restored in the last sector of device $LINTEL_DEVICE"
        else
            warning "lintel signature has not been restored in the last sector of device $LINTEL_DEVICE"
        fi
    fi

else

    : erase

    if does_device_have_b_lintel_signature
    then
        if .is_action_confirmed "Do you want to erase lintel signature in the zeroth sector of device $LINTEL_DEVICE?"
        then
            erase_b_lintel_signature $LINTEL_DEVICE
            report "lintel signature has been erased from the zeroth sector of device $LINTEL_DEVICE"
        else
            warning "lintel signature has not been erased from the zeroth sector of device $LINTEL_DEVICE"
        fi
    fi

    if does_device_have_e_lintel_signature
    then
        if .is_action_confirmed "Do you want to erase lintel signature in the last sector of device $LINTEL_DEVICE?"
        then
            erase_e_lintel_signature $LINTEL_DEVICE
            report "lintel signature has been erased from the last sector of device $LINTEL_DEVICE"
        else
            warning "lintel signature has not been erased from the last sector of device $LINTEL_DEVICE"
        fi
    fi

fi

.exit 0
