#!/bin/bash +x

declare SCRIPT_DIR=$( dirname $( readlink -f $0 ) )
source $SCRIPT_DIR/include

# Check environment
declare LINTEL_PACK_BINDIR=/opt/mcst/lintel/bin
if does_tool_exist bcd_write $LINTEL_PACK_BINDIR
then
    declare ONE_DISK_MODE_SUPPORTED=
fi

function print_help()
{
    if declare -p __SHOW_ACTION__ >& /dev/null
    then
        printf "\nPlace lintel to device\n\n"
    else
        printf "\n"
    fi
    printf "Usage:\n"
    printf "$( readlink -f $0 ) [options]\n\n"
    printf "Options:\n"
    printf "\t-h, -H, --help                   print this help\n"
    printf "\t-I <lintel image>                set lintel image name (by default searching disk-file in /opt/mcst/lintel/bin)\n"
    printf "\t-D <lintel storage device>       set device to place lintel (by default trying /dev/hda, /dev/hdc)\n"
    printf "\t-F                               ignore MBR and existing device partitions table (ignore disk content mode),\n"
    printf "\t                                 some data on disk may be lost\n"
    if declare -p ONE_DISK_MODE_SUPPORTED >& /dev/null
    then
        printf "\t-E                               place lintel to the end of disk (one disk mode), 4GiB of continuous free space\n"
        printf "\t                                 is needed at least\n"
    fi
    printf "\n"
    return 0
}

# Parse options
declare LINTEL_IMAGE=""
declare LINTEL_DEVICE=""
declare IS_PLACE_FORCED=""
declare IS_ONE_DISK_MODE=""
declare PARSE_OPTIONS_SECTION=
while [ $# != 0 ]
do
    case $1 in
        -h | -H | --help )
            __SHOW_ACTION__= print_help
            exit 0
        ;;
        -I )
            shift
            test $1 || error "no argument for -I option"
            test -f $1 || error "invalid argument for -I option"
            LINTEL_IMAGE="$1"
            shift
            continue
        ;;
        -D )
            shift
            test $1 || error "no argument for -D option"
            test -b $1 || error "invalid argument for -D option"
            LINTEL_DEVICE="$1"
            shift
            continue
        ;;
        -F )
            IS_PLACE_FORCED=1
            shift
            continue
        ;;
        -E )
            if declare -p ONE_DISK_MODE_SUPPORTED >& /dev/null
            then
                IS_ONE_DISK_MODE=1
                shift
                continue
            fi
        ;;
    esac
    error "unsupported option $1"
done
unset -v PARSE_OPTIONS_SECTION

# If LINTEL_DEVICE is not set try to use default device
if [ -z "${LINTEL_DEVICE:-""}" ]
then
    if test -b /dev/hda
    then
        LINTEL_DEVICE=/dev/hda
    else
        if test -b /dev/hdc
        then
            LINTEL_DEVICE=/dev/hdc
        else
            error "can not find IDE device to place lintel image"
        fi
    fi
fi
__INFO__= info "device $LINTEL_DEVICE will be used"

# If LINTEL_IMAGE is not set try to use default image
if [ -z "${LINTEL_IMAGE:-""}" ]
then
    for LINTEL_IMAGE in $LINTEL_PACK_BINDIR/lintel_e*.disk
    do
        if test -e $LINTEL_IMAGE
        then
            break
        else
            error "cat not find lintel image in $LINTEL_PACK_BINDIR"
        fi
    done
fi
__INFO__= info "lintel image $LINTEL_IMAGE will be used"

examine_device $LINTEL_DEVICE

if [ -n "${IS_ONE_DISK_MODE:-""}" ]
then
    info "one disk mode is selected (-E)"
    examine_image $LINTEL_IMAGE
fi

# Final checks
if [ -z "${IS_PLACE_FORCED:-""}" ]
then

    if does_device_have_broken_pt
    then
        error "device $LINTEL_DEVICE has broken partition table, use -F option to ignore"
    fi

    if [ -z "${IS_ONE_DISK_MODE:-""}" ]
    then
        if does_device_have_b_overlap
        then
            error "device $LINTEL_DEVICE has master boot record, use -F option to ignore"
        fi
    else
        if does_device_have_e_overlap
        then
            error "device $LINTEL_DEVICE has no enough free space for lintel, use -F option to ignore"
        fi
    fi

fi
if [ -n "${IS_ONE_DISK_MODE:-""}" ]
then

    if is_device_too_small_for_one_disk_mode
    then
        error "device $LINTEL_DEVICE is too small for one-disk-mode (<4GiB)"
    fi

    if is_device_too_large_for_one_disk_mode
    then
        error "device $LINTEL_DEVICE is too large for one-disk_mode (>2TiB)"
    fi

fi


# Place lintel
if [ -n "${IS_ONE_DISK_MODE:-""}" ]
then

    if ! does_device_support_one_disk_mode
    then
        warning "One disk mode (option -E) for $LINTEL_IMAGE is not supported in booting firmware"
    fi

    if does_device_have_b_lintel_signature
    then
        erase_b_lintel_signature $LINTEL_DEVICE
    fi

    if does_device_have_e_overlap
    then
        delete_overlapped_partitions $LINTEL_DEVICE
    fi

else

    if does_device_have_e_lintel_signature
    then
        erase_e_lintel_signature $LINTEL_DEVICE
    fi

    : all existing partitions will be erased in MBR automatically

fi
if [ -n "${IS_PLACE_FORCED:-""}" ]
then
    info "ignore disk content mode is selected (-F)"
fi

declare STATUS=
if [ -n "${IS_ONE_DISK_MODE:-""}" ]
then
    if does_device_support_old_one_disk_mode
    then
        $LINTEL_PACK_BINDIR/bcd_write -I $LINTEL_IMAGE -D $LINTEL_DEVICE -E -O -F >& /dev/null
        STATUS=$?
    else
        $LINTEL_PACK_BINDIR/bcd_write -I $LINTEL_IMAGE -D $LINTEL_DEVICE -E -F >& /dev/null
        STATUS=$?
    fi
else
    .dd_write if=$LINTEL_IMAGE of=$LINTEL_DEVICE >& /dev/null
    STATUS=$?
fi

if [ $STATUS == 0 ]
then
    if [ -n "${IS_ONE_DISK_MODE:-""}" ]
    then
        report "lintel image $LINTEL_IMAGE has been successfully placed at the end of $LINTEL_DEVICE"
    else
        report "lintel image $LINTEL_IMAGE has been successfully placed at the beginning of $LINTEL_DEVICE"
    fi
else
    error "can not place image $LINTEL_IMAGE to device $LINTEL_DEVICE"
fi

.exit 0
