#!/bin/bash

declare SCRIPT_DIR=$( dirname $( readlink -f $0 ) )
source $SCRIPT_DIR/include

# Check environment
declare LINTEL_PACK_DIR=/opt/mcst/lintel
declare LINTEL_PACK_BINDIR=$LINTEL_PACK_DIR/bin
check_tool tar

function print_help()
{
    if declare -p __SHOW_ACTION__ >& /dev/null
    then
        printf "\nGet lintel log from device and archivate it\n\n"
    else
        printf "\n"
    fi
    printf "Usage:\n"
    printf "$( readlink -f $0 ) [options]\n\n"
    printf "Options:\n"
    printf "\t-h, -H, --help                   print this help\n"
    printf "\t-D <lintel storage device>       set device to get lintel log from (by default trying /dev/hda, /dev/hdc)\n"
    printf "\t-O <output directory>            set directory to place lintel log archive (by default using $LINTEL_PACK_DIR),\n"
    printf "\t                                 at least 1.5GiB of free disk space in output directory is required\n"
    return 0
}

# Parse options
declare LINTEL_DEVICE=""
declare OUTPUT_DIR=""
declare PARSE_OPTIONS_SECTION=
while [ $# != 0 ]
do
    case $1 in
        -h | -H | --help )
            __SHOW_ACTION__= print_help
            exit 0
        ;;
        -D )
            shift
            test $1 || error "no argument for -D option"
            test -b $1 || error "invalid argument for -D option"
            LINTEL_DEVICE="$1"
        ;;
        -O )
            shift
            test $1 || error "no argument for -O option"
            test -d $1 || error "invalid argument for -O option"
            test -w $1 || error "non-writable output directory $1"
            OUTPUT_DIRECTORY="$1"
        ;;
        * )
            error "unsupported option $1"
        ;;
    esac
    shift
done
unset -v PARSE_OPTIONS_SECTION

# If LINTEL_DEVICE is not set try to use default device
if [ -z "${LINTEL_DEVICE:-""}" ]
then
    if test -e /dev/hda
    then
        LINTEL_DEVICE=/dev/hda
    else
        if test -e /dev/hdc
        then
            LINTEL_DEVICE=/dev/hdc
        else
            error "can not find IDE device to get lintel log from"
        fi
    fi
fi
__INFO__= info "device $LINTEL_DEVICE will be explored"

# If OUTPUT_DIRECTORY is not set use default place
if [ -z "${OUTPUT_DIRECTORY:-""}" ]
then
    OUTPUT_DIRECTORY=$LINTEL_PACK_DIR
fi
__INFO__= info "output directory $OUTPUT_DIRECTORY will be used"

examine_device $LINTEL_DEVICE

# Check old lintel log archives
declare LINTEL_LOG=$OUTPUT_DIRECTORY/lintel_log
declare B_LOG_SUFFIX=""
declare E_LOG_SUFFIX=""
if does_device_have_b_lintel_signature && does_device_have_e_lintel_signature
then
    warning "two lintels found on device $LINTEL_DEVICE"
    B_LOG_SUFFIX=b
    E_LOG_SUFFIX=e
fi
declare LOG_IDX=0
declare OLD_ARCHIVES_NUM=0
declare OLD_ARCHIVES_WARNING_LIMIT=5
for (( ; ; LOG_IDX = ( LOG_IDX == 0 ) ? 2 : LOG_IDX + 1 ))
do
    for LOG_SUFFIX in "" $B_LOG_SUFFIX $E_LOG_SUFFIX
    do
        if test -e ${LINTEL_LOG}${LOG_SUFFIX:+_$LOG_SUFFIX}${LOG_IDX/#0/}.txz
        then
            OLD_ARCHIVES_NUM=$(( OLD_ARCHIVES_NUM + 1 ))
            continue 2
        fi
    done
    LOG_IDX=${LOG_IDX/#0/}
    break
done
if [ $OLD_ARCHIVES_NUM -ge $OLD_ARCHIVES_WARNING_LIMIT ]
then
    warning "a plenty of old lintel log archives found in directory $OUTPUT_DIRECTORY"
fi

if test -e $LINTEL_LOG
then
    rm -rf $LINTEL_LOG
fi
declare FREE_SPACE_ERROR_LIMIT=$(( 1024 * 1024 * 1024 * 3 / 2 ))            # 1.5GiB
declare LINTEL_LOG_ERROR_LIMIT=$(( 1024 * 1024 * 1024 + 50 * 1024 * 1024 )) # 1GiB + 50 MiB

declare LINTEL_LOG_ARCHIVE=
declare FREE_LBA_OFFSET=$(( 0xc ))
declare LINTEL_FULL_SIZE=
if does_device_have_b_lintel_signature
then

    if does_device_have_b_bcd_signature
    then

        if ! is_enough_free_space_available $OUTPUT_DIRECTORY $FREE_SPACE_ERROR_LIMIT
        then
            error "no enough free disk space available in output directory $OUTPUT_DIRECTORY"
        fi

        info "reading lintel log ..."
        if does_tool_exist bcd_read $LINTEL_PACK_BINDIR
        then
            $LINTEL_PACK_BINDIR/bcd_read -D $LINTEL_DEVICE -I $LINTEL_LOG >& /dev/null
        else
            LINTEL_FULL_SIZE=$( .dd_read if=$LINTEL_DEVICE bs=1 skip=$(( SECTOR_SIZE + FREE_LBA_OFFSET )) count=8 2> /dev/null | hexdump -e '"%08x "' | awk '{ print "0x" $2 $1 }' )
            .dd_read if=$LINTEL_DEVICE of=$LINTEL_LOG bs=$SECTOR_SIZE count=$(( LINTEL_FULL_SIZE )) >& /dev/null
        fi
        STATUS=$?
        if is_lintel_log_too_big $LINTEL_LOG $LINTEL_LOG_ERROR_LIMIT
        then
            warning "lintel ${B_LOG_SUFFIX:+$B_LOG_SUFFIX-}log file is larger than expected"
            .debug_error "lintel ${B_LOG_SUFFIX:+$B_LOG_SUFFIX-}log file is larger than expected"
        fi

        if [ $STATUS == 0 ]
        then

            LINTEL_LOG_ARCHIVE=$LINTEL_LOG${B_LOG_SUFFIX:+_$B_LOG_SUFFIX}$LOG_IDX.txz
            __INFO__= info "archivating lintel log ..."
            tar -cJf $LINTEL_LOG_ARCHIVE $LINTEL_LOG >& /dev/null
            if [ $? == 0 ]
            then
                report "lintel ${B_LOG_SUFFIX:+$B_LOG_SUFFIX-}log archive $LINTEL_LOG_ARCHIVE has been successfully created"
                __INFO__= info "lintel ${B_LOG_SUFFIX:+$B_LOG_SUFFIX-}log archive $LINTEL_LOG_ARCHIVE has been successfully created"
                rm $LINTEL_LOG
            else
                rm $LINTEL_LOG
                rm $LINTEL_LOG_ARCHIVE
                error "lintel ${B_LOG_SUFFIX:+$B_LOG_SUFFIX-}log archivation has failed"
            fi

        else

            rm $LINTEL_LOG
            error "lintel ${B_LOG_SUFFIX:+$B_LOG_SUFFIX-}log reading from device $LINTEL_DEVICE has failed"

        fi

    else

        warning "detected old lintel on device $LINTEL_DEVICE with no log section"

    fi

fi

if does_device_have_e_lintel_signature
then

    if does_device_have_e_bcd_signature
    then

    if ! is_enough_free_space_available $OUTPUT_DIRECTORY $FREE_SPACE_ERROR_LIMIT
    then
        error "no enough free disk space available in output directory $OUTPUT_DIRECTORY"
    fi

    info "reading lintel log ..."
    if does_tool_exist bcd_read $LINTEL_PACK_BINDIR
    then
        $LINTEL_PACK_BINDIR/bcd_read -D $LINTEL_DEVICE -I $LINTEL_LOG -E >& /dev/null
    else
        declare LINTEL_DEVICE_SIZE=$( blockdev --getsz $LINTEL_DEVICE )
        LINTEL_FULL_SIZE=$( .dd_read if=$LINTEL_DEVICE bs=1 skip=$(( LINTEL_DEVICE_SIZE * SECTOR_SIZE - ODM_DISK_MINSIZE + SECTOR_SIZE + FREE_LBA_OFFSET )) count=8 2> /dev/null | hexdump -e '"%08x "' | awk '{ print "0x" $2 $1 }' )
        .dd_read if=$LINTEL_DEVICE of=$LINTEL_LOG bs=$SECTOR_SIZE skip=$(( LINTEL_DEVICE_SIZE - ODM_DISK_MINSIZE / SECTOR_SIZE )) count=$(( LINTEL_FULL_SIZE )) >& /dev/null
    fi
    STATUS=$?
    if is_lintel_log_too_big $LINTEL_LOG $LINTEL_LOG_ERROR_LIMIT
    then
        warning "lintel ${E_LOG_SUFFIX:+$E_LOG_SUFFIX-}log file is larger than expected"
        .debug_error "lintel ${E_LOG_SUFFIX:+$E_LOG_SUFFIX-}log file is larger than expected" 
    fi

    if [ $STATUS == 0 ]
    then

        LINTEL_LOG_ARCHIVE=$LINTEL_LOG${E_LOG_SUFFIX:+_$E_LOG_SUFFIX}$LOG_IDX.txz
        __INFO__= info "archivating lintel log ..."
        tar -cJf $LINTEL_LOG_ARCHIVE $LINTEL_LOG >& /dev/null
        if [ $? == 0 ]
        then
            report "lintel ${E_LOG_SUFFIX:+$E_LOG_SUFFIX-}log archive $LINTEL_LOG_ARCHIVE has been successfully created"
            __INFO__= info "lintel ${E_LOG_SUFFIX:+$E_LOG_SUFFIX-}log archive $LINTEL_LOG_ARCHIVE has been successfully created"
            rm $LINTEL_LOG
        else
            rm $LINTEL_LOG
            rm $LINTEL_LOG_ARCHIVE
            error "lintel ${E_LOG_SUFFIX:+$E_LOG_SUFFIX-}log archivation has failed"
        fi

    else

        rm $LINTEL_LOG
        error "lintel ${E_LOG_SUFFIX:+$E_LOG_SUFFIX-}log reading from device $LINTEL_DEVICE has failed"

    fi

    else

        warning "detected old lintel on device $LINTEL_DEVICE with no log section"

    fi

fi

.exit 0
